package com.ismart.ib86.app;

import android.Manifest;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Handler;
import android.view.Window;
import android.view.WindowInsets;
import android.view.WindowInsetsController;
import android.os.Build;
import android.view.View;
import android.util.Log;
import android.widget.Toast;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.ismart.ib86.app.R;
import com.ismart.ib86.feature.serial.SerialPort.ASR5515Protocol;
import com.ismart.ib86.feature.serial.SerialPort.SerialPortHelper;
import com.ismart.ib86.feature.gpio.GpioManager;
import com.ismart.ib86.feature.serial.SerialPort.ASR5515DeviceManager;
import com.ismart.ib86.feature.serial.SerialPort.ASR5515Protocol;
import com.ismart.ib86.common.utils.LogManager;
import com.ismart.ib86.feature.gpio.GpioTest;

public class MainActivity extends AppCompatActivity {
    private static final String TAG = "MainActivity";
    private static final String DEVICE_PATH = "/dev/ttyS2";
    private static final int BAUD_RATE = 921600;
    
    // 权限请求码
    private static final int PERMISSION_REQUEST_CODE = 1001;
    // 需要请求的权限
    private static final String[] REQUIRED_PERMISSIONS = {
        Manifest.permission.READ_EXTERNAL_STORAGE,
        Manifest.permission.WRITE_EXTERNAL_STORAGE
    };

    private static final int BT_POWER_EN = 75;
    private ASR5515DeviceManager deviceManager;
    private Handler handler = new Handler();
    private GpioTest gpioTest;
    private GpioManager gpioManager;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // 检查并请求必要的权限
        checkAndRequestPermissions();

        // 隐藏状态栏和导航栏
        hideSystemBars();
        // 初始化设备管理器
        initDeviceManager();

        // 延迟2秒后开始测试，确保设备管理器初始化完成
        handler.postDelayed(new Runnable() {
            @Override
            public void run() {
                startProtocolTests();
            }
        }, 2000);
    }

    private void initDeviceManager() {
        //打开5515电源
        gpioManager = GpioManager.getInstance();
        gpioManager.exportGpio(BT_POWER_EN);
        gpioManager.setDirection(BT_POWER_EN, GpioManager.DIRECTION_OUT);
        gpioManager.setValue(BT_POWER_EN, GpioManager.VALUE_HIGH);

        deviceManager = new ASR5515DeviceManager(this, DEVICE_PATH, BAUD_RATE);
        // 设置设备检查回调
        deviceManager.setDeviceCheckListener(deviceInfo -> {
            if (deviceInfo != null) {
                LogManager.d(TAG, "设备检查成功: " + deviceInfo.toString());
            } else {
                LogManager.e(TAG, "设备检查失败");
            }
        });

        // 设置设备控制回调
        deviceManager.setDeviceControlListener(new ASR5515DeviceManager.DeviceControlListener() {
            @Override
            public void onDeviceRestartResponse(boolean success) {
                if (success) {
                    LogManager.d(TAG, "设备重启成功");
                } else {
                    LogManager.e(TAG, "设备重启失败");
                }
            }
        });

        // 设置日志控制回调
        deviceManager.setLogControlListener(new ASR5515DeviceManager.LogControlListener() {
            @Override
            public void onLogControlResponse(boolean success) {
                if (success) {
                    LogManager.d(TAG, "日志控制设置成功");
                } else {
                    LogManager.e(TAG, "日志控制设置失败");
                }
            }
        });

        // 设置蓝牙版本回调
        deviceManager.setBtVersionListener(version -> {
            if (version != null) {
                LogManager.d(TAG, "蓝牙版本: " + version);
            } else {
                LogManager.e(TAG, "获取蓝牙版本失败");
            }
        });

        // 设置蓝牙升级回调
        deviceManager.setBtUpgradeListener(response -> 
            LogManager.d(TAG, "蓝牙升级: " + response.toString())
        );

        // 设置主机状态回调
        deviceManager.setHostStatusListener(new ASR5515DeviceManager.HostStatusListener() {
            @Override
            public void onHostStatusResponse(ASR5515Protocol.HostStatusResponse response) {
                LogManager.d(TAG, "主机状态: " + response.status);
            }
        });
        deviceManager.setBootBinCheckListener(new ASR5515DeviceManager.BootBinCheckListener() {
            @Override
            public void onBootBinCheckResponse(ASR5515Protocol.BootBinCheckResponse response) {
                LogManager.d(TAG, "Boot bin检查: " + response.toString());
            }
        });
        deviceManager.setCollectFreqSetListener(new ASR5515DeviceManager.CollectFreqSetListener() {
            @Override
            public void onCollectFreqSetResponse(ASR5515Protocol.CollectFreqSetResponse response) {
                LogManager.d(TAG, "采集频率设置: " + response.toString());
            }
        });

        // 设置心率血压血氧自动测量回调
        deviceManager.setHrBpBoAutoMeasureListener(new ASR5515DeviceManager.HrBpBoAutoMeasureListener() {
            @Override
            public void onHrBpBoAutoMeasureResponse(ASR5515Protocol.HrBpBoAutoMeasureResponse response) {
                LogManager.d(TAG, "HrBpBo自动测量: " + response.toString());
            }
        });

        // 设置心率血压血氧手动测量回调
        deviceManager.setHrBpBoManualMeasureListener(new ASR5515DeviceManager.HrBpBoManualMeasureListener() {
            @Override
            public void onHrBpBoManualMeasureResponse(ASR5515Protocol.HrBpBoManualMeasureResponse response) {
                LogManager.d(TAG, "HrBpBo手动测量: " + response.toString());
            }
        });

        // 设置动态测量回调
        deviceManager.setDynamicMeasureListener(response -> 
            LogManager.d(TAG, "动态测量: " + response.toString())
        );
        
        // 设置佩戴检测回调
        deviceManager.setWearDetectionListener(response -> 
            LogManager.d(TAG, "佩戴检测: " + response.isWearing)
        );
    }

    private void startProtocolTests() {
        LogManager.d(TAG, "开始协议测试...");
        
//        handler.postDelayed(new Runnable() {
//            @Override
//            public void run() {
//                deviceManager.checkDevice();
//            }
//        }, 1000);
//
//        handler.postDelayed(new Runnable() {
//            @Override
//            public void run() {
//                deviceManager.queryBtVersion();
//            }
//        }, 2000);

        handler.postDelayed(new Runnable() {
            @Override
            public void run() {
                deviceManager.sendWearDetectionRequest();
            }
        }, 1000);

//        handler.postDelayed(new Runnable() {
//            @Override
//            public void run() {
//                deviceManager.sendWearDetectionRequest();
//            }
//        }, 4000);
    }

    @Override
protected void onDestroy() {
    super.onDestroy();
    handler.removeCallbacksAndMessages(null);
    
    // 移至后台线程执行耗时操作，避免ANR
    new Thread(() -> {
        if (deviceManager != null) {
            deviceManager.syncHostStatus(false);
            deviceManager.close();
        }
        
        // 释放GPIO测试资源
        if (gpioTest != null) {
            gpioTest.release();
            gpioTest = null;
        }
    }).start();
}

private void hideSystemBars() {
    Window window = getWindow();
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
        WindowInsetsController insetsController = window.getInsetsController();
        if (insetsController != null) {
            // 隐藏状态栏和导航栏 (API 30+)
            insetsController.hide(WindowInsets.Type.statusBars() | WindowInsets.Type.navigationBars());
            insetsController.setSystemBarsBehavior(WindowInsetsController.BEHAVIOR_SHOW_TRANSIENT_BARS_BY_SWIPE);
        }
    } else {
        // 兼容旧版本 (API < 30)
        View decorView = window.getDecorView();
        int flags = View.SYSTEM_UI_FLAG_FULLSCREEN
                | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY;
        decorView.setSystemUiVisibility(flags);
    }
}

@Override
public void onWindowFocusChanged(boolean hasFocus) {
    super.onWindowFocusChanged(hasFocus);
    if (hasFocus) {
        hideSystemBars();
    }
}

/**
 * 检查并请求必要的权限
 */
private void checkAndRequestPermissions() {
    // 检查是否已经有所需的权限
    boolean allPermissionsGranted = true;
    for (String permission : REQUIRED_PERMISSIONS) {
        if (ContextCompat.checkSelfPermission(this, permission) != PackageManager.PERMISSION_GRANTED) {
            allPermissionsGranted = false;
            break;
        }
    }

    // 如果没有所有权限，则请求它们
    if (!allPermissionsGranted) {
        ActivityCompat.requestPermissions(this, REQUIRED_PERMISSIONS, PERMISSION_REQUEST_CODE);
    }
}

/**
 * 处理权限请求结果
 */
@Override
public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
    super.onRequestPermissionsResult(requestCode, permissions, grantResults);
    
    if (requestCode == PERMISSION_REQUEST_CODE) {
        boolean allGranted = true;
        for (int result : grantResults) {
            if (result != PackageManager.PERMISSION_GRANTED) {
                allGranted = false;
                break;
            }
        }
        
        if (allGranted) {
            // 所有权限都已授予，可以继续操作
            LogManager.d(TAG, "所有必要权限已授予");

        } else {
            // 有些权限被拒绝，显示提示并可能影响功能
            Toast.makeText(this, "需要所有请求的权限才能正常运行应用", Toast.LENGTH_LONG).show();
            LogManager.e(TAG, "部分权限被拒绝，可能影响应用功能");
        }
    }
}

}