package com.ismart.ib86.feature.gpio;

import com.ismart.ib86.common.utils.LogManager;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * GPIO管理类，提供GPIO的基本操作
 * 纯Java实现，直接操作/sys/class/gpio/目录下的文件
 */
public class GpioManager {
    private static final String TAG = "GpioManager";
    private static volatile GpioManager instance;
    
    // GPIO方向常量
    public static final int DIRECTION_IN = 0;
    public static final int DIRECTION_OUT = 1;
    
    // GPIO值常量
    public static final int VALUE_LOW = 0;
    public static final int VALUE_HIGH = 1;
    
    // GPIO路径常量
    private static final String GPIO_PATH = "/sys/class/gpio/";
    private static final String GPIO_EXPORT_PATH = GPIO_PATH + "export";
    private static final String GPIO_UNEXPORT_PATH = GPIO_PATH + "unexport";
    
    // 文件缓存，避免频繁打开关闭文件
    private final Map<String, File> fileCache = new HashMap<>();
    
    // 最后的错误信息
    private String lastErrorMsg = "";

    private GpioManager() {
        init();
    }

    public static GpioManager getInstance() {
        if (instance == null) {
            synchronized (GpioManager.class) {
                if (instance == null) {
                    instance = new GpioManager();
                }
            }
        }
        return instance;
    }
    
    /**
     * 初始化GPIO管理器
     * @return 初始化是否成功
     */
    private boolean init() {
        LogManager.d(TAG, "初始化GPIO管理器");

        // 检查GPIO目录是否存在
        File gpioDir = new File(GPIO_PATH);
        if (!gpioDir.exists() || !gpioDir.isDirectory()) {
            setLastError("GPIO sysfs接口不可用: 目录不存在");
            return false;
        }
        // 检查export和unexport文件的访问权限
        File exportFile = new File(GPIO_EXPORT_PATH);
        File unexportFile = new File(GPIO_UNEXPORT_PATH);

        if (!exportFile.exists() || !exportFile.canWrite()) {
            setLastError("无法访问文件 " + GPIO_EXPORT_PATH + ": 文件不存在或无写入权限");
            return false;
        }
        
        if (!unexportFile.exists() || !unexportFile.canWrite()) {
            setLastError("无法访问文件 " + GPIO_UNEXPORT_PATH + ": 文件不存在或无写入权限");
            return false;
        }
        
        LogManager.d(TAG, "GPIO管理器初始化成功");
        return true;
    }

    /**
     * 导出GPIO引脚
     * @param pin GPIO引脚号
     * @return 操作是否成功
     */
    public boolean exportGpio(int pin) {
        //LogManager.d(TAG, "导出GPIO: " + pin);

        // 检查GPIO是否已经导出
        String gpioPath = GPIO_PATH + "gpio" + pin;
        File gpioDir = new File(gpioPath);
        
        if (gpioDir.exists() && gpioDir.isDirectory()) {
            //LogManager.d(TAG, "GPIO " + pin + " 已经导出");
            return true;
        }
        
        // 导出GPIO
        if (!writeToFile(GPIO_EXPORT_PATH, String.valueOf(pin))) {
            return false;
        }
        
        // 等待GPIO目录创建完成
        for (int i = 0; i < 10; i++) {
            if (gpioDir.exists() && gpioDir.isDirectory()) {
                //LogManager.d(TAG, "GPIO " + pin + " 导出成功");
                return true;
            }
            
            try {
                TimeUnit.MILLISECONDS.sleep(100);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                LogManager.e(TAG, "等待GPIO导出被中断: " + e.getMessage());
            }
        }
        
        setLastError("GPIO " + pin + " 导出超时");
        return false;
    }

    /**
     * 取消导出GPIO引脚
     * @param pin GPIO引脚号
     * @return 操作是否成功
     */
    public boolean unexportGpio(int pin) {
        LogManager.d(TAG, "取消导出GPIO: " + pin);
        
        // 检查GPIO是否已经导出
        String gpioPath = GPIO_PATH + "gpio" + pin;
        File gpioDir = new File(gpioPath);
        
        if (!gpioDir.exists() || !gpioDir.isDirectory()) {
            LogManager.d(TAG, "GPIO " + pin + " 未导出");
            return true;
        }
        
        // 从缓存中移除相关文件
        String directionPath = gpioPath + "/direction";
        String valuePath = gpioPath + "/value";
        
        synchronized (fileCache) {
            fileCache.remove(directionPath);
            fileCache.remove(valuePath);
        }
        
        // 取消导出GPIO
        if (!writeToFile(GPIO_UNEXPORT_PATH, String.valueOf(pin))) {
            return false;
        }
        
        // 等待GPIO目录删除完成
        for (int i = 0; i < 10; i++) {
            if (!gpioDir.exists()) {
                LogManager.d(TAG, "GPIO " + pin + " 取消导出成功");
                return true;
            }
            
            try {
                TimeUnit.MILLISECONDS.sleep(100);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                LogManager.e(TAG, "等待GPIO取消导出被中断: " + e.getMessage());
            }
        }
        
        setLastError("GPIO " + pin + " 取消导出超时");
        return false;
    }

    /**
     * 设置GPIO方向
     * @param pin GPIO引脚号
     * @param direction 方向（DIRECTION_IN或DIRECTION_OUT）
     * @return 操作是否成功
     */
    public boolean setDirection(int pin, int direction) {
        if (direction != DIRECTION_IN && direction != DIRECTION_OUT) {
            LogManager.e(TAG, "无效的方向值: " + direction);
            return false;
        }
        
        //LogManager.d(TAG, "设置GPIO " + pin + " 方向为: " + (direction == DIRECTION_IN ? "IN" : "OUT"));
        
        // 检查GPIO是否已导出
        String gpioPath = GPIO_PATH + "gpio" + pin;
        File gpioDir = new File(gpioPath);
        
        if (!gpioDir.exists() || !gpioDir.isDirectory()) {
            setLastError("GPIO " + pin + " 未导出");
            return false;
        }
        
        // 设置方向
        String directionPath = gpioPath + "/direction";
        String dirStr = (direction == DIRECTION_IN) ? "in" : "out";
        
        if (!writeToFile(directionPath, dirStr)) {
            return false;
        }
        
        //LogManager.d(TAG, "GPIO " + pin + " 方向设置为 " + dirStr + " 成功");
        return true;
    }

    /**
     * 设置GPIO值
     * @param pin GPIO引脚号
     * @param value 值（VALUE_LOW或VALUE_HIGH）
     * @return 操作是否成功
     */
    public boolean setValue(int pin, int value) {
        if (value != VALUE_LOW && value != VALUE_HIGH) {
            LogManager.e(TAG, "无效的值: " + value);
            return false;
        }
        
        //LogManager.d(TAG, "设置GPIO " + pin + " 值为: " + value);
        
        // 检查GPIO是否已导出
        String gpioPath = GPIO_PATH + "gpio" + pin;
        File gpioDir = new File(gpioPath);
        
        if (!gpioDir.exists() || !gpioDir.isDirectory()) {
            setLastError("GPIO " + pin + " 未导出");
            return false;
        }
        
        // 设置值
        String valuePath = gpioPath + "/value";
        String valueStr = String.valueOf(value);
        
        if (!writeToFile(valuePath, valueStr)) {
            return false;
        }
        
        //LogManager.d(TAG, "GPIO " + pin + " 值设置为 " + valueStr + " 成功");
        return true;
    }

    /**
     * 获取GPIO值
     * @param pin GPIO引脚号
     * @return GPIO值，-1表示获取失败
     */
    public int getValue(int pin) {
        //LogManager.d(TAG, "获取GPIO " + pin + " 值");
        
        // 检查GPIO是否已导出
        String gpioPath = GPIO_PATH + "gpio" + pin;
        File gpioDir = new File(gpioPath);
        
        if (!gpioDir.exists() || !gpioDir.isDirectory()) {
            setLastError("GPIO " + pin + " 未导出");
            return -1;
        }
        
        // 读取值
        String valuePath = gpioPath + "/value";
        String value = readFromFile(valuePath);
        
        if (value == null || value.isEmpty()) {
            return -1;
        }
        
        // 解析值
        try {
            int result = Integer.parseInt(value.trim());
            //LogManager.d(TAG, "GPIO " + pin + " 值读取为 " + result);
            return result;
        } catch (NumberFormatException e) {
            setLastError("无法解析GPIO " + pin + " 的值: " + e.getMessage());
            return -1;
        }
    }

    /**
     * 释放资源
     */
    public void release() {
        LogManager.d(TAG, "释放GPIO资源");
        
        // 清空文件缓存
        synchronized (fileCache) {
            fileCache.clear();
        }
        
        instance = null;
    }
    
    /**
     * 获取最后的错误信息
     * @return 错误信息
     */
    public String getLastError() {
        return lastErrorMsg;
    }
    
    /**
     * 设置最后的错误信息
     * @param message 错误信息
     */
    private void setLastError(String message) {
        lastErrorMsg = message;
        LogManager.e(TAG, message);
    }
    
    /**
     * 写入文件
     * @param path 文件路径
     * @param content 内容
     * @return 是否成功
     */
    private boolean writeToFile(String path, String content) {
        BufferedWriter writer = null;
        
        try {
            writer = new BufferedWriter(new FileWriter(path));
            writer.write(content);
            writer.flush();
            return true;
        } catch (IOException e) {
            setLastError("写入文件 " + path + " 失败: " + e.getMessage());
            return false;
        } finally {
            if (writer != null) {
                try {
                    writer.close();
                } catch (IOException e) {
                    LogManager.e(TAG, "关闭文件写入流失败: " + e.getMessage());
                }
            }
        }
    }
    
    /**
     * 从文件读取
     * @param path 文件路径
     * @return 文件内容，失败返回null
     */
    private String readFromFile(String path) {
        BufferedReader reader = null;
        
        try {
            reader = new BufferedReader(new FileReader(path));
            return reader.readLine();
        } catch (IOException e) {
            setLastError("读取文件 " + path + " 失败: " + e.getMessage());
            return null;
        } finally {
            if (reader != null) {
                try {
                    reader.close();
                } catch (IOException e) {
                    LogManager.e(TAG, "关闭文件读取流失败: " + e.getMessage());
                }
            }
        }
    }
}