package com.ismart.ib86.feature.gpio;

import com.ismart.ib86.common.utils.LogManager;

/**
 * GPIO测试类，用于测试GPIO功能
 */
public class GpioTest {
    private static final String TAG = "GpioTest";
    
    // 测试用的GPIO引脚号，根据实际硬件调整
    private static final int TEST_GPIO_PIN = 26;  // 示例引脚号，需要根据实际硬件调整
    
    private GpioManager gpioManager;
    
    public GpioTest() {
        gpioManager = GpioManager.getInstance();
        LogManager.d(TAG, "GpioTest初始化完成");
    }
    
    /**
     * 开始GPIO测试
     */
    public void startTest() {
        LogManager.d(TAG, "开始GPIO测试...");
        
        try {
            // 测试1: 导出GPIO引脚
            testExportGpio();
            
            // 测试2: 设置GPIO方向
            testSetDirection();
            
            // 测试3: 设置和获取GPIO值
            testSetAndGetValue();
            
            // 测试4: 取消导出GPIO引脚
            testUnexportGpio();
            
            LogManager.d(TAG, "GPIO测试完成");
        } catch (Exception e) {
            LogManager.e(TAG, "GPIO测试异常: " + e.getMessage());
        }
    }
    
    /**
     * 测试导出GPIO引脚
     */
    private void testExportGpio() {
        LogManager.d(TAG, "测试导出GPIO引脚 " + TEST_GPIO_PIN);
        boolean result = gpioManager.exportGpio(TEST_GPIO_PIN);
        if (result) {
            LogManager.d(TAG, "GPIO引脚 " + TEST_GPIO_PIN + " 导出成功");
        } else {
            LogManager.e(TAG, "GPIO引脚 " + TEST_GPIO_PIN + " 导出失败");
        }
    }
    
    /**
     * 测试设置GPIO方向
     */
    private void testSetDirection() {
        LogManager.d(TAG, "测试设置GPIO方向为输出");
        boolean result = gpioManager.setDirection(TEST_GPIO_PIN, GpioManager.DIRECTION_OUT);
        if (result) {
            LogManager.d(TAG, "GPIO引脚 " + TEST_GPIO_PIN + " 方向设置为输出成功");
        } else {
            LogManager.e(TAG, "GPIO引脚 " + TEST_GPIO_PIN + " 方向设置为输出失败");
        }
    }
    
    /**
     * 测试设置和获取GPIO值
     */
    private void testSetAndGetValue() {
        // 设置为高电平
        LogManager.d(TAG, "测试设置GPIO值为高电平");
        boolean setHighResult = gpioManager.setValue(TEST_GPIO_PIN, GpioManager.VALUE_HIGH);
        if (setHighResult) {
            LogManager.d(TAG, "GPIO引脚 " + TEST_GPIO_PIN + " 设置为高电平成功");
        } else {
            LogManager.e(TAG, "GPIO引脚 " + TEST_GPIO_PIN + " 设置为高电平失败");
        }
        
        // 读取值
        int highValue = gpioManager.getValue(TEST_GPIO_PIN);
        LogManager.d(TAG, "GPIO引脚 " + TEST_GPIO_PIN + " 当前值: " + highValue);
        
        // 等待一段时间
        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {
            LogManager.e(TAG, "线程睡眠被中断: " + e.getMessage());
        }
        
        // 设置为低电平
        LogManager.d(TAG, "测试设置GPIO值为低电平");
        boolean setLowResult = gpioManager.setValue(TEST_GPIO_PIN, GpioManager.VALUE_LOW);
        if (setLowResult) {
            LogManager.d(TAG, "GPIO引脚 " + TEST_GPIO_PIN + " 设置为低电平成功");
        } else {
            LogManager.e(TAG, "GPIO引脚 " + TEST_GPIO_PIN + " 设置为低电平失败");
        }
        
        // 读取值
        int lowValue = gpioManager.getValue(TEST_GPIO_PIN);
        LogManager.d(TAG, "GPIO引脚 " + TEST_GPIO_PIN + " 当前值: " + lowValue);
    }
    
    /**
     * 测试取消导出GPIO引脚
     */
    private void testUnexportGpio() {
        LogManager.d(TAG, "测试取消导出GPIO引脚 " + TEST_GPIO_PIN);
        boolean result = gpioManager.unexportGpio(TEST_GPIO_PIN);
        if (result) {
            LogManager.d(TAG, "GPIO引脚 " + TEST_GPIO_PIN + " 取消导出成功");
        } else {
            LogManager.e(TAG, "GPIO引脚 " + TEST_GPIO_PIN + " 取消导出失败");
        }
    }
    
    /**
     * 释放资源
     */
    public void release() {
        LogManager.d(TAG, "释放GPIO测试资源");
        if (gpioManager != null) {
            gpioManager.release();
        }
    }
}