package com.ismart.ib86.feature.serial.SerialPort;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;

import com.ismart.ib86.common.utils.LogManager;
import com.ismart.ib86.feature.gpio.GpioManager;

/**
 * ASR-5515设备管理器
 * 用于管理与ASR-5515设备的通信
 */
public class ASR5515DeviceManager {
    private static final String TAG = "ASR5515DeviceManager";
    private static final int GPIO_BT_WAKE = 33;  // 蓝牙唤醒GPIO
    
    private final SerialPortHelper serialPortHelper;
    private final Handler mainHandler;
    private final GpioManager gpioManager;
      // 细分回调实例
    private DeviceCheckListener deviceCheckListener;
    private DeviceControlListener deviceControlListener;
    private LogControlListener logControlListener;
    private BtVersionListener btVersionListener;
    private BtUpgradeListener btUpgradeListener;
    private HostStatusListener hostStatusListener;
    private BootBinCheckListener bootBinCheckListener;
    private CollectFreqSetListener collectFreqSetListener;
    private HrBpBoAutoMeasureListener hrBpBoAutoMeasureListener;
    private HrBpBoManualMeasureListener hrBpBoManualMeasureListener;
    private DynamicMeasureListener dynamicMeasureListener;
    private WearDetectionListener wearDetectionListener;

    // 细分回调接口
    public interface DeviceCheckListener {
        void onDeviceCheckResponse(ASR5515Protocol.DeviceInfo deviceInfo);
    }

    public interface DeviceControlListener {
        void onDeviceRestartResponse(boolean success);
    }

    public interface LogControlListener {
        void onLogControlResponse(boolean success);
    }

    public interface BtVersionListener {
        void onBtVersionResponse(String version);
    }

    public interface BtUpgradeListener {
        void onBtUpgradeResponse(ASR5515Protocol.BtUpgradeResponse response);
    }

    public interface HostStatusListener {
        void onHostStatusResponse(ASR5515Protocol.HostStatusResponse response);
    }

    public interface BootBinCheckListener {
        void onBootBinCheckResponse(ASR5515Protocol.BootBinCheckResponse response);
    }

    public interface CollectFreqSetListener {
        void onCollectFreqSetResponse(ASR5515Protocol.CollectFreqSetResponse response);
    }

    public interface HrBpBoAutoMeasureListener {
        void onHrBpBoAutoMeasureResponse(ASR5515Protocol.HrBpBoAutoMeasureResponse response);
    }

    public interface HrBpBoManualMeasureListener {
        void onHrBpBoManualMeasureResponse(ASR5515Protocol.HrBpBoManualMeasureResponse response);
    }

    public interface DynamicMeasureListener {
        void onDynamicMeasureResponse(ASR5515Protocol.DynamicMeasureResponse response);
    }
    
    public interface WearDetectionListener {
        void onWearDetectionResponse(ASR5515Protocol.WearDetectionResponse response);
    }

    /**
     * 构造函数
     * @param context 上下文
     * @param devicePath 设备路径
     * @param baudRate 波特率
     */
    public ASR5515DeviceManager(Context context, String devicePath, int baudRate) {
        serialPortHelper = new SerialPortHelper(devicePath, baudRate);
        mainHandler = new Handler(Looper.getMainLooper());
        gpioManager = GpioManager.getInstance();
        serialPortHelper.setSimulationMode(false);
        // 初始化串口
        boolean result = serialPortHelper.open();
        if (!result) {
            LogManager.e(TAG, "Failed to open serial port: " + devicePath);
        }
        
        // 设置设备状态回调
        serialPortHelper.setDeviceCheckCallback(new SerialPortHelper.DeviceCheckCallback() {
            @Override
            public void onDeviceCheckResponse(final ASR5515Protocol.DeviceInfo deviceInfo) {
                if (deviceCheckListener != null) {
                    mainHandler.post(() -> deviceCheckListener.onDeviceCheckResponse(deviceInfo));
                }
            }
        });

        serialPortHelper.setDeviceControlCallback(new SerialPortHelper.DeviceControlCallback() {
            @Override
            public void onDeviceRestartResponse(final boolean success) {
                if (deviceControlListener != null) {
                    mainHandler.post(() -> deviceControlListener.onDeviceRestartResponse(success));
                }
            }
        });

        serialPortHelper.setLogControlCallback(new SerialPortHelper.LogControlCallback() {
            @Override
            public void onLogControlResponse(final boolean success) {
                if (logControlListener != null) {
                    mainHandler.post(() -> logControlListener.onLogControlResponse(success));
                }
            }
        });

        serialPortHelper.setBtVersionCallback(new SerialPortHelper.BtVersionCallback() {
            @Override
            public void onBtVersionResponse(final String version) {
                if (btVersionListener != null) {
                    mainHandler.post(() -> btVersionListener.onBtVersionResponse(version));
                }
            }
        });

        serialPortHelper.setBtUpgradeCallback(new SerialPortHelper.BtUpgradeCallback() {
            @Override
            public void onBtUpgradeResponse(final ASR5515Protocol.BtUpgradeResponse response) {
                if (btUpgradeListener != null) {
                    mainHandler.post(() -> btUpgradeListener.onBtUpgradeResponse(response));
                }
            }
        });

        serialPortHelper.setHostStatusCallback(new SerialPortHelper.HostStatusCallback() {
            @Override
            public void onHostStatusResponse(final ASR5515Protocol.HostStatusResponse response) {
                if (hostStatusListener != null) {
                    mainHandler.post(() -> hostStatusListener.onHostStatusResponse(response));
                }
            }
        });
        
        serialPortHelper.setBootBinCheckCallback(new SerialPortHelper.BootBinCheckCallback() {
            @Override
            public void onBootBinCheckResponse(final ASR5515Protocol.BootBinCheckResponse response) {
                if (bootBinCheckListener != null) {
                    mainHandler.post(() -> bootBinCheckListener.onBootBinCheckResponse(response));
                }
            }
        });
        
        serialPortHelper.setCollectFreqSetCallback(new SerialPortHelper.CollectFreqSetCallback() {
            @Override
            public void onCollectFreqSetResponse(final ASR5515Protocol.CollectFreqSetResponse response) {
                if (collectFreqSetListener != null) {
                    mainHandler.post(() -> collectFreqSetListener.onCollectFreqSetResponse(response));
                }
            }
        });

        serialPortHelper.setHrBpBoAutoMeasureCallback(new SerialPortHelper.HrBpBoAutoMeasureCallback() {
            @Override
            public void onHrBpBoAutoMeasureResponse(ASR5515Protocol.HrBpBoAutoMeasureResponse response) {
                if (hrBpBoAutoMeasureListener != null) {
                    mainHandler.post(() -> hrBpBoAutoMeasureListener.onHrBpBoAutoMeasureResponse(response));
                }
            }
        });

        serialPortHelper.setHrBpBoManualMeasureCallback(new SerialPortHelper.HrBpBoManualMeasureCallback() {
            @Override
            public void onHrBpBoManualMeasureResponse(ASR5515Protocol.HrBpBoManualMeasureResponse response) {
                if (hrBpBoManualMeasureListener != null) {
                    mainHandler.post(() -> hrBpBoManualMeasureListener.onHrBpBoManualMeasureResponse(response));
                }
            }
        });

        serialPortHelper.setDynamicMeasureCallback(new SerialPortHelper.DynamicMeasureCallback() {
            @Override
            public void onDynamicMeasureResponse(ASR5515Protocol.DynamicMeasureResponse response) {
                if (dynamicMeasureListener != null) {
                    mainHandler.post(() -> dynamicMeasureListener.onDynamicMeasureResponse(response));
                }
            }
        });
        
        serialPortHelper.setWearDetectionCallback(new SerialPortHelper.WearDetectionCallback() {
            @Override
            public void onWearDetectionResponse(ASR5515Protocol.WearDetectionResponse response) {
                if (wearDetectionListener != null) {
                    mainHandler.post(() -> wearDetectionListener.onWearDetectionResponse(response));
                }
            }
        });
    }
    
    /**
     * 设置设备检查监听器
     */
    public void setDeviceCheckListener(DeviceCheckListener listener) {
        this.deviceCheckListener = listener;
    }

    /**
     * 设置设备控制监听器
     */
    public void setDeviceControlListener(DeviceControlListener listener) {
        this.deviceControlListener = listener;
    }

    public void setLogControlListener(LogControlListener listener) {
        this.logControlListener = listener;
    }

    /**
     * 设置蓝牙版本监听器
     */
    public void setBtVersionListener(BtVersionListener listener) {
        this.btVersionListener = listener;
    }

    /**
     * 设置蓝牙升级监听器
     */
    public void setBtUpgradeListener(BtUpgradeListener listener) {
        this.btUpgradeListener = listener;
    }

    /**
     * 设置主机状态监听器
     */
    public void setHostStatusListener(HostStatusListener listener) {
        this.hostStatusListener = listener;
    }

    /**
     * 设置心率血压血氧自动测量监听器
     */
    public void setHrBpBoAutoMeasureListener(HrBpBoAutoMeasureListener listener) {
        this.hrBpBoAutoMeasureListener = listener;
    }

    /**
     * 设置心率血压血氧手动测量监听器
     */
    public void setHrBpBoManualMeasureListener(HrBpBoManualMeasureListener listener) {
        this.hrBpBoManualMeasureListener = listener;
    }

    /**
     * 设置动态测量监听器
     */
    public void setDynamicMeasureListener(DynamicMeasureListener listener) {
        this.dynamicMeasureListener = listener;
    }
    
    /**
     * 设置佩戴检测监听器
     */
    public void setWearDetectionListener(WearDetectionListener listener) {
        this.wearDetectionListener = listener;
    }
    
    /**
     * 设置Boot Bin检查监听器
     */
    public void setBootBinCheckListener(BootBinCheckListener listener) {
        this.bootBinCheckListener = listener;
    }
    
    /**
     * 设置采集频率设置监听器
     */
    public void setCollectFreqSetListener(CollectFreqSetListener listener) {
        this.collectFreqSetListener = listener;
    }
    
    /**
     * 检查设备状态
     */
    public void checkDevice() {
        wakeupDevice();
        serialPortHelper.checkDevice();
    }
    
    /**
     * 重启设备
     */
    public void restartDevice() {
        wakeupDevice();
        serialPortHelper.restartDevice();
    }
    
    /**
     * 控制日志输出
     * @param enable 是否启用
     */
    public void setLogControl(boolean enable) {
        wakeupDevice();
        serialPortHelper.setLogControl(enable);
    }
    
    /**
     * 查询蓝牙版本
     */
    public void queryBtVersion() {
        wakeupDevice();
        serialPortHelper.queryBtVersion();
    }
    
    /**
     * 发送蓝牙升级请求
     */
    public void upgradeBt() {
        wakeupDevice();
        serialPortHelper.upgradeBt();
    }
    
    /**
     * 发送主机状态同步请求
     * @param screenOn 屏幕状态（true：亮屏，false：灭屏）
     */
    public void syncHostStatus(boolean screenOn) {
        wakeupDevice();
        serialPortHelper.syncHostStatus(screenOn);
    }
    
    /**
     * 关闭设备
     */
    public void close() {
        serialPortHelper.close();
    }

    /**
     * 发送boot bin检查请求
     */
    public void checkBootBin() {
        wakeupDevice();
        serialPortHelper.checkBootBin();
    }

    /**
     * 设置采集频率
     * @param len 长度
     * @param freq 频率值
     */
    public void setCollectFreq(byte len, byte freq) {
        wakeupDevice();
        serialPortHelper.setCollectFreq(len, freq);
    }

    /**
     * 发送HrBpBo自动测量请求
     */
    public void startHrBpBoAutoMeasure() {
        wakeupDevice();
        serialPortHelper.startHrBpBoAutoMeasure();
    }

    /**
     * 发送HrBpBo手动测量请求
     */
    public void startHrBpBoManualMeasure() {
        wakeupDevice();
        serialPortHelper.startHrBpBoManualMeasure();
    }

    /**
     * 发送动态测量请求
     */
    public void startDynamicMeasure() {
        wakeupDevice();
        serialPortHelper.startDynamicMeasure();
    }
    
    /**
     * 发送佩戴检测请求
     */
    public void sendWearDetectionRequest() {
        wakeupDevice();
        serialPortHelper.sendWearDetectionRequest();
    }
    
    /**
     * 唤醒设备
     * 通过GPIO33唤醒蓝牙，ASR休眠时会自动发送'w'字符
     */
    public void wakeupDevice() {
        try {
            // 配置GPIO33为输出模式
            gpioManager.exportGpio(GPIO_BT_WAKE);
            gpioManager.setDirection(GPIO_BT_WAKE, GpioManager.DIRECTION_OUT);
            
            // 拉低GPIO33
            gpioManager.setValue(GPIO_BT_WAKE, 0);
            
            // 等待30ms
            Thread.sleep(30);
            
            // 拉高GPIO33
            gpioManager.setValue(GPIO_BT_WAKE, 1);
            
        } catch (Exception e) {
            LogManager.e(TAG, "Failed to wakeup bluetooth: " + e.getMessage());
        }
    }
}