package com.ismart.ib86.feature.serial.SerialPort;

import static com.ismart.ib86.feature.serial.SerialPort.SerialPortHelper.bytesToHexString;

import android.os.Build;
import android.util.Log;

import com.ismart.ib86.common.utils.LogManager;

import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.HexFormat;

public class ASR5515Protocol {
    private static final String TAG = "ASR5515Protocol";
    // 设备检查回调接口
    public interface DeviceCheckCallback {
        void onDeviceCheckResponse(DeviceInfo deviceInfo);
    }
    
    // 设备控制回调接口
    public interface DeviceControlCallback {
        void onDeviceRestartResponse(boolean success);
        void onLogControlResponse(boolean success);
    }
    
    // 蓝牙版本回调接口
    public interface BtVersionCallback {
        void onBtVersionResponse(String version);
    }
    
    // 蓝牙升级回调接口
    public interface BtUpgradeCallback {
        void onBtUpgradeResponse(BtUpgradeResponse response);
    }
    
    // 主机状态回调接口
    public interface HostStatusCallback {
        void onHostStatusResponse(HostStatusResponse response);
    }
    
    // Boot Bin检查回调接口
    public interface BootBinCheckCallback {
        void onBootBinCheckResponse(BootBinCheckResponse response);
    }
    
    // 采集频率设置回调接口
    public interface CollectFreqSetCallback {
        void onCollectFreqSetResponse(CollectFreqSetResponse response);
    }
    
    // HrBpBo自动测量回调接口
    public interface HrBpBoAutoMeasureCallback {
        void onHrBpBoAutoMeasureResponse(HrBpBoAutoMeasureResponse response);
    }
    
    // HrBpBo手动测量回调接口
    public interface HrBpBoManualMeasureCallback {
        void onHrBpBoManualMeasureResponse(HrBpBoManualMeasureResponse response);
    }
    
    // 动态测量回调接口
    public interface DynamicMeasureCallback {
        void onDynamicMeasureResponse(DynamicMeasureResponse response);
    }
    
    // 佩戴检测回调接口
    public interface WearDetectionCallback {
        void onWearDetectionResponse(WearDetectionResponse response);
    }
    private static final byte FRAME_START = (byte) '[';
    private static final byte FRAME_END = (byte) ']';

    public static class DeviceInfo {
        public byte sn;          // 序列号
        public byte len;         // 长度
        public byte heartRate;   // 心率
        public byte heartRateId; // 心率ID
        public byte stepCount;   // 计步
        public byte stepCountId; // 计步ID
        public String deviceId;  // 设备ID
        public String hardwareVersion; // 硬件版本
        public String softwareVersion; // 软件版本
        public String status;    // 设备状态

        public DeviceInfo(byte sn, byte len, byte heartRate, byte heartRateId, byte stepCount, byte stepCountId, 
                         String deviceId, String hardwareVersion, String softwareVersion, String status) {
            this.sn = sn;
            this.len = len;
            this.heartRate = heartRate;
            this.heartRateId = heartRateId;
            this.stepCount = stepCount;
            this.stepCountId = stepCountId;
            this.deviceId = deviceId;
            this.hardwareVersion = hardwareVersion;
            this.softwareVersion = softwareVersion;
            this.status = status;
        }

        @Override
        public String toString() {
            return String.format("设备信息:\n设备ID: %s\n硬件版本: %s\n软件版本: %s\n状态: %s",
                    deviceId, hardwareVersion, softwareVersion, status);
        }
    }

    // 第一个parseDeviceCheckResponse方法被删除，保留文件末尾的更详细实现

    public static class RebootResponse {
        public byte sn;
        public byte status;

        public RebootResponse(byte sn, byte status) {
            this.sn = sn;
            this.status = status;
        }
    }

    public static RebootResponse parseRebootResponse(Frame frame) {
        if (frame == null || frame.command != Commands.CMD_DEVICE_REBOOT_RESP || frame.data.length < 2) {
            return null;
        }
        return new RebootResponse(frame.data[0], frame.data[1]);
    }

    public static class TimeSyncResponse {
        public byte sn;
        public byte status;

        public TimeSyncResponse(byte sn, byte status) {
            this.sn = sn;
            this.status = status;
        }
    }

    public static TimeSyncResponse parseTimeSyncResponse(Frame frame) {
        if (frame == null || frame.command != Commands.CMD_TIME_SYNC_RESP || frame.data.length < 2) {
            return null;
        }
        return new TimeSyncResponse(frame.data[0], frame.data[1]);
    }

    public static class LogControlResponse {
        public byte sn;
        public byte status;

        public LogControlResponse(byte sn, byte status) {
            this.sn = sn;
            this.status = status;
        }
    }

    public static LogControlResponse parseLogControlResponse(Frame frame) {
        if (frame == null || frame.command != Commands.CMD_LOG_CONTROL_RESP || frame.data.length < 2) {
            return null;
        }
        return new LogControlResponse(frame.data[0], frame.data[1]);
    }

    // 创建设备重启请求帧
    public static byte[] createRebootRequest(byte sn) {
        return createFrame(Commands.CMD_DEVICE_REBOOT, new byte[]{sn, 0});
    }

    // 创建时间同步请求帧
    public static byte[] createTimeSyncRequest(byte sn, byte len, int timestamp) {
        byte[] data = new byte[6];
        data[0] = sn;
        data[1] = len;
        // 将timestamp转换为4字节数组
        data[2] = (byte) (timestamp >> 24);
        data[3] = (byte) (timestamp >> 16);
        data[4] = (byte) (timestamp >> 8);
        data[5] = (byte) timestamp;
        return createFrame(Commands.CMD_TIME_SYNC, data);
    }

    // 创建日志打印控制请求帧
    public static byte[] createLogControlRequest(byte sn, byte len, byte enable) {
        return createFrame(Commands.CMD_LOG_CONTROL, new byte[]{sn, len, enable});
    }

    public static class Commands {
        public static final short CMD_CHECK_DEVICE = 0x0065;      // 设备检查请求
        public static final short CMD_CHECK_DEVICE_RESP = 0x0066; // 设备检查响应
        public static final short CMD_DEVICE_REBOOT = 0x0067;     // 设备重启请求
        public static final short CMD_DEVICE_REBOOT_RESP = 0x0068;// 设备重启响应
        public static final short CMD_TIME_SYNC = 0x0069;         // 时间同步请求
        public static final short CMD_TIME_SYNC_RESP = 0x006A;    // 时间同步响应
        public static final short CMD_RESTART_DEVICE = 0x0067;    // 设备重启请求
        public static final short CMD_RESTART_DEVICE_RESP = 0x0068; // 设备重启响应
        public static final short CMD_LOG_CONTROL = 0x006B;       // 日志打印控制请求
        public static final short CMD_LOG_CONTROL_RESP = 0x006C;  // 日志打印控制响应
        public static final short CMD_BT_VERSION = 0x006D;        // 蓝牙版本请求
        public static final short CMD_BT_VERSION_RESP = 0x006E;   // 蓝牙版本响应
        public static final short CMD_BT_UPGRADE = 0x0071;        // 本地、蓝牙升级请求
        public static final short CMD_BT_UPGRADE_RESP = 0x0078;   // 本地、蓝牙升级响应
        public static final short CMD_HOST_STATUS = 0x0079;       // 主机状态同步请求
        public static final short CMD_HOST_STATUS_RESP = 0x007A;  // 主机状态同步响应
        public static final short CMD_HOST_STATUS_BOOT_RESP = 0x007C; //主机状态,启动状态
        // 新增命令
        public static final short CMD_BOOT_BIN_CHECK = 0x007D;    // boot bin检查请求 [125 0 0]
        public static final short CMD_BOOT_BIN_CHECK_RESP = 0x007E; // boot bin检查响应 [126 sn len 0/1]
        public static final short CMD_COLLECT_FREQ_SET = 0x00C9;  // 采集频率设置 [201 0 len 32]
        public static final short CMD_COLLECT_FREQ_SET_RESP = 0x00CA; // 采集频率设置响应 [202 sn 0]
        public static final short CMD_HRBPBO_AUTO_MEASURE = 0x00CB; // HrBpBo自动测量 [203 0 0]
        public static final short CMD_HRBPBO_AUTO_MEASURE_RESP = 0x00CC; // HrBpBo自动测量响应 [204 sn 0]
        public static final short CMD_HRBPBO_MANUAL_MEASURE = 0x00CD; // HrBpBo手动测量 [205 0 0]
        public static final short CMD_HRBPBO_MANUAL_MEASURE_RESP = 0x00CE; // HrBpBo手动测量响应 [206 sn 0]
        public static final short CMD_DYNAMIC_MEASURE = 0x00CF;   // 动态测量 [207 0 0]
        public static final short CMD_DYNAMIC_MEASURE_RESP = 0x00D0; // 动态测量响应 [208 sn 0]
        public static final short CMD_WEAR_DETECTION = 0x0105;    // 佩戴检测查询 [261 0 0]
        public static final short CMD_WEAR_DETECTION_RESP = 0x0106; // 佩戴检测响应 [262 sn len 0/1]
    }

    public static class Frame {
        public short command;  // 修改为short类型以支持2字节命令
        public byte[] data;
        public byte[] remainingData; // 解析后剩余的数据

        public Frame(short command, byte[] data) {
            this.command = command;
            this.data = data;
        }
    }

    public static byte[] createDeviceCheckRequest() {
        // 发送格式：[101 0 0]
        byte[] data = new byte[2];  // 两个0
        data[0] = 0;
        data[1] = 0;
        return createFrame(Commands.CMD_CHECK_DEVICE, data);
    }

    public static byte[] createFrame(short command, byte[] data) {
        LogManager.d(TAG, "LEN :" + data.length);
        int frameLength = 1 + 2 + 2 + 2 + data.length + 1; // start + command(2bytes) + sn(2bytes) + len(2bytes) + data + end
        ByteBuffer buffer = ByteBuffer.allocate(frameLength);
        
        buffer.put(FRAME_START);           // 添加帧起始符'['
        buffer.put((byte)(command & 0xFF)); // 命令字节低位
        buffer.put((byte)((command >> 8) & 0xFF)); // 命令字节高位
        buffer.put((byte)0x00); // sn
        buffer.put((byte)0x00); //sn
        buffer.put((byte)(data.length & 0xFF)); // len
        buffer.put((byte)((data.length >> 8) & 0xFF)); //len
        if(data.length > 0){
            buffer.put(data);                 // 数据部分
        }
        buffer.put(FRAME_END);            // 添加帧结束符']'
        
        return buffer.array();
    }

    public static Frame parseFrame(byte[] data) {
        if (data == null || data.length < 8) { // 最小帧长度：head(1) + type(2) + sn(2) + len(2) + end(1)
            return null;
        }

        // 查找帧起始符'['
        int startIndex = -1;
        for (int i = 0; i < data.length; i++) {
            if (data[i] == FRAME_START) {
                startIndex = i;
                break;
            }
        }

        if (startIndex == -1 || startIndex + 7 >= data.length) {
            return null;
        }

        // 从起始符后开始查找帧结束符']'
        int endIndex = -1;
        for (int i = startIndex + 7; i < data.length; i++) {
            if (data[i] == FRAME_END) {
                endIndex = i;
                break;
            }
        }

        if (endIndex == -1) {
            return null;
        }

        try {
            // 提取type（2字节）
            short type = (short)(((data[startIndex + 2] & 0xFF) << 8) | (data[startIndex + 1] & 0xFF));
            
            // 提取sn（2字节）
            short sn = (short)(((data[startIndex + 4] & 0xFF) << 8) | (data[startIndex + 3] & 0xFF));
            
            // 提取len（2字节）
            short len = (short)(((data[startIndex + 6] & 0xFF) << 8) | (data[startIndex + 5] & 0xFF));

            // 提取数据部分（如果有的话）
            int dataLength = endIndex - (startIndex + 7); // 7 = head(1) + type(2) + sn(2) + len(2)
            //LogManager.d(TAG, "len: "+ len + " dataLength: " + dataLength + " end data" + data[endIndex]);
            byte[] frameData;
            if (dataLength > 0) {
                frameData = new byte[dataLength];
                System.arraycopy(data, startIndex + 7, frameData, 0, dataLength);
//                LogManager.d(TAG,"data :"+ bytesToHexString(frameData));
            } else {
                frameData = new byte[0];
            }

            // 创建帧对象
            Frame frame = new Frame(type, frameData);

            // 如果还有剩余数据，保存到remainingData
            if (endIndex + 1 < data.length) {
                frame.remainingData = new byte[data.length - endIndex - 1];
                System.arraycopy(data, endIndex + 1, frame.remainingData, 0, frame.remainingData.length);
            }

            return frame;
        } catch (Exception e) {
            LogManager.e(TAG, "Parse frame error: " + e.getMessage());
            return null;
        }
    }

    public static DeviceInfo parseDeviceCheckResponse(Frame frame) {
        if (frame == null || frame.command != Commands.CMD_CHECK_DEVICE_RESP || frame.data.length < 7) {
            return null;
        }

        try {
            ByteBuffer buffer = ByteBuffer.wrap(frame.data);
            
            // 解析响应数据：[sn len 心率 ID 计步 ID 设备ID]
            byte sn = buffer.get();          // 序列号
            byte len = buffer.get();         // 长度
            byte heartRate = buffer.get();   // 心率
            byte heartRateId = buffer.get(); // 心率ID
            byte stepCount = buffer.get();   // 计步
            byte stepCountId = buffer.get(); // 计步ID
            byte rawDeviceId = buffer.get(); // 设备ID
            
            // 将deviceId转换为十六进制字符串
            String deviceId = String.format("%02X", rawDeviceId);
            
            // 硬件版本和软件版本可能不在帧数据中，设置默认值
            String hardwareVersion = "V1.0";
            String softwareVersion = "V1.0";
            String status = "正常"; // 默认状态
            
            // 如果帧数据中包含更多信息，可以继续解析
            if (frame.data.length > 7) {
                // 这里需要根据实际协议格式解析硬件版本、软件版本和状态
                byte statusByte = frame.data[7];
                status = parseStatus(statusByte);
            }

            return new DeviceInfo(sn, len, heartRate, heartRateId, stepCount, stepCountId, deviceId, hardwareVersion, softwareVersion, status);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    private static String parseStatus(byte statusByte) {
        switch(statusByte) {
            case 0:
                return "正常";
            case 1:
                return "异常";
            case 2:
                return "离线";
            default:
                return "未知状态(" + statusByte + ")";
        }
    }

    // 创建蓝牙版本请求帧
    public static byte[] createBtVersionRequest() {
        return createFrame(Commands.CMD_BT_VERSION, new byte[]{});
    }

    // 创建设备重启请求帧
    public static byte[] createDeviceRestartRequest() {
        return createFrame(Commands.CMD_RESTART_DEVICE, new byte[]{});
    }

    // 解析设备重启响应帧
    public static boolean parseDeviceRestartResponse(Frame frame) {
        return frame != null && frame.command == Commands.CMD_RESTART_DEVICE_RESP;
    }

    public static String parseBtVersionResponse(Frame frame) {
        if (frame == null || frame.command != Commands.CMD_BT_VERSION_RESP || frame.data.length == 0) {
            return null;
        }
        StringBuilder sb = new StringBuilder();
        for (byte b : frame.data) {
            // 只保留可打印的 ASCII 字符（32-126）
            if (b >= 32 && b <= 126) {
                sb.append((char) b);
            }
        }
        try {
            return sb.toString();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    // 创建本地、蓝牙升级请求帧
    public static byte[] createBtUpgradeRequest() {
        // 发送格式：[113 0 0]
        byte[] data = new byte[2];  // 两个0
        data[0] = 0;
        data[1] = 0;
        return createFrame(Commands.CMD_BT_UPGRADE, data);
    }

    // 解析本地、蓝牙升级响应帧
    public static class BtUpgradeResponse {
        public byte sn;
        public byte len;
        public String data;

        public BtUpgradeResponse(byte sn, byte len, String data) {
            this.sn = sn;
            this.len = len;
            this.data = data;
        }
    }

    public static BtUpgradeResponse parseBtUpgradeResponse(Frame frame) {
        if (frame == null || frame.command != Commands.CMD_BT_UPGRADE_RESP || frame.data.length < 2) {
            return null;
        }

        try {
            byte sn = frame.data[0];
            byte len = frame.data[1];
            
            // 提取数据部分（如果有的话）
            String data = "";
            if (frame.data.length > 2) {
                byte[] dataBytes = new byte[frame.data.length - 2];
                System.arraycopy(frame.data, 2, dataBytes, 0, dataBytes.length);
                data = new String(dataBytes, StandardCharsets.UTF_8);
            }
            
            return new BtUpgradeResponse(sn, len, data);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    // 创建主机状态同步请求帧
    public static byte[] createHostStatusRequest(boolean screenOn) {
        // 发送格式：[121 0 len 0/1]，其中0/1表示屏幕状态（灭/亮）
        byte[] data = new byte[3];
        data[0] = 0;  // 保留字节
        data[1] = 1;
        data[2] = (byte)(screenOn ? 1 : 0);  // 1表示亮屏，0表示灭屏
        return createFrame(Commands.CMD_HOST_STATUS, data);
    }

    // 创建boot bin检查请求帧 [125 0 0]
    public static byte[] createBootBinCheckRequest() {
        byte[] data = new byte[2];
        data[0] = 0;
        data[1] = 0;
        return createFrame(Commands.CMD_BOOT_BIN_CHECK, data);
    }

    // 创建采集频率设置请求帧 [201 0 len 32]
    public static byte[] createCollectFreqSetRequest(byte len, byte freq) {
        byte[] data = new byte[3];
        data[0] = 0;  // 保留字节
        data[1] = len;
        data[2] = freq;
        return createFrame(Commands.CMD_COLLECT_FREQ_SET, data);
    }

    // 创建HrBpBo自动测量请求帧 [203 0 0]
    public static byte[] createHrBpBoAutoMeasureRequest() {
        byte[] data = new byte[2];
        data[0] = 0;
        data[1] = 0;
        return createFrame(Commands.CMD_HRBPBO_AUTO_MEASURE, data);
    }

    // 创建HrBpBo手动测量请求帧 [205 0 0]
    public static byte[] createHrBpBoManualMeasureRequest() {
        byte[] data = new byte[2];
        data[0] = 0;
        data[1] = 0;
        return createFrame(Commands.CMD_HRBPBO_MANUAL_MEASURE, data);
    }

    // 创建动态测量请求帧 [207 0 0]
    public static byte[] createDynamicMeasureRequest() {
        byte[] data = new byte[2];
        data[0] = 0;
        data[1] = 0;
        return createFrame(Commands.CMD_DYNAMIC_MEASURE, data);
    }

    // 解析主机状态同步响应帧
    public static class HostStatusResponse {
        public byte sn;
        public byte status;

        public HostStatusResponse(byte sn, byte status) {
            this.sn = sn;
            this.status = status;
        }
    }

    public static HostStatusResponse parseHostStatusResponse(Frame frame) {
        if (frame == null || frame.data.length < 2) {
            return null;
        }
        
        return new HostStatusResponse(frame.data[0], frame.data[1]);
    }

    // BootBinCheck响应类
    public static class BootBinCheckResponse {
        public byte sn;
        public byte len;
        public byte status;

        public BootBinCheckResponse(byte sn, byte len, byte status) {
            this.sn = sn;
            this.len = len;
            this.status = status;
        }
    }

    public static BootBinCheckResponse parseBootBinCheckResponse(Frame frame) {
        if (frame == null || frame.command != Commands.CMD_BOOT_BIN_CHECK_RESP || frame.data.length < 3) {
            return null;
        }
        return new BootBinCheckResponse(frame.data[0], frame.data[1], frame.data[2]);
    }

    // CollectFreqSet响应类
    public static class CollectFreqSetResponse {
        public byte sn;
        public byte status;

        public CollectFreqSetResponse(byte sn, byte status) {
            this.sn = sn;
            this.status = status;
        }
    }

    public static CollectFreqSetResponse parseCollectFreqSetResponse(Frame frame) {
        if (frame == null || frame.command != Commands.CMD_COLLECT_FREQ_SET_RESP || frame.data.length < 2) {
            return null;
        }
        return new CollectFreqSetResponse(frame.data[0], frame.data[1]);
    }

    // HrBpBoAutoMeasure响应类
    public static class HrBpBoAutoMeasureResponse {
        public byte sn;
        public byte status;

        public HrBpBoAutoMeasureResponse(byte sn, byte status) {
            this.sn = sn;
            this.status = status;
        }
    }

    public static HrBpBoAutoMeasureResponse parseHrBpBoAutoMeasureResponse(Frame frame) {
        if (frame == null || frame.command != Commands.CMD_HRBPBO_AUTO_MEASURE_RESP || frame.data.length < 2) {
            return null;
        }
        return new HrBpBoAutoMeasureResponse(frame.data[0], frame.data[1]);
    }

    // HrBpBoManualMeasure响应类
    public static class HrBpBoManualMeasureResponse {
        public byte sn;
        public byte status;

        public HrBpBoManualMeasureResponse(byte sn, byte status) {
            this.sn = sn;
            this.status = status;
        }
    }

    public static HrBpBoManualMeasureResponse parseHrBpBoManualMeasureResponse(Frame frame) {
        if (frame == null || frame.command != Commands.CMD_HRBPBO_MANUAL_MEASURE_RESP || frame.data.length < 2) {
            return null;
        }
        return new HrBpBoManualMeasureResponse(frame.data[0], frame.data[1]);
    }

    // DynamicMeasure响应类
    public static class DynamicMeasureResponse {
        public byte sn;
        public byte status;

        public DynamicMeasureResponse(byte sn, byte status) {
            this.sn = sn;
            this.status = status;
        }
    }

    public static DynamicMeasureResponse parseDynamicMeasureResponse(Frame frame) {
        if (frame == null || frame.command != Commands.CMD_DYNAMIC_MEASURE_RESP || frame.data.length < 2) {
            return null;
        }
        return new DynamicMeasureResponse(frame.data[0], frame.data[1]);
    }

    // 创建佩戴检测查询请求帧 [261 0 0]
    public static byte[] createWearDetectionRequest() {
        return createFrame(Commands.CMD_WEAR_DETECTION, new byte[]{});
    }

    // 佩戴检测响应类
    public static class WearDetectionResponse {
        public boolean isWearing; // true表示已佩戴，false表示未佩戴

        public WearDetectionResponse( boolean isWearing) {
            this.isWearing = isWearing;
        }

        @Override
        public String toString() {
            return "WearDetectionResponse{" +
                    ", isWearing=" + isWearing +
                    '}';
        }
    }

    // 解析佩戴检测响应帧 [262 sn len 0/1]
    public static WearDetectionResponse parseWearDetectionResponse(Frame frame) {
        if (frame == null || frame.command != Commands.CMD_WEAR_DETECTION_RESP ) {
            return null;
        }
        boolean isWearing = frame.data[0] == 1;
        return new WearDetectionResponse(isWearing);
    }
}