package com.ismart.ib86.feature.serial.SerialPort;

import android.serialport.SerialPort;

import com.ismart.ib86.common.utils.LogManager;

import java.io.File;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.Arrays;
import java.util.Objects;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;

public class SerialPortHelper {
    private final static String TAG = "SerialPortHelper";
    private SerialPort mSerialPort;
    private String mPortName;
    private int mBaudRate;
    private final BlockingQueue<byte[]> sendQueue = new LinkedBlockingQueue<>();
    private final BlockingQueue<byte[]> receiveQueue = new LinkedBlockingQueue<>(100);
    private Thread sendThread;
    private Thread receiveThread;
    private Thread processThread;
    private volatile boolean isRunning;
    private SendCallback sendCallback;
    private ReceiveCallback receiveCallback;
    private DeviceCheckCallback deviceCheckCallback;
    private DeviceControlCallback deviceControlCallback;
    private LogControlCallback logControlCallback;
    private BtVersionCallback btVersionCallback;
    private BtUpgradeCallback btUpgradeCallback;
    private HostStatusCallback hostStatusCallback;
    private BootBinCheckCallback bootBinCheckCallback;
    private CollectFreqSetCallback collectFreqSetCallback;
    private HrBpBoAutoMeasureCallback hrBpBoAutoMeasureCallback;
    private HrBpBoManualMeasureCallback hrBpBoManualMeasureCallback;
    private DynamicMeasureCallback dynamicMeasureCallback;
    private WearDetectionCallback wearDetectionCallback;
    private boolean isSimulationMode = false;
    private final ByteBuffer frameBuffer = ByteBuffer.allocate(1024);

    public interface SendCallback {
        void onSendSuccess(byte[] data);

        void onSendFailed(byte[] data, Exception e);
    }

    public interface ReceiveCallback {
        void onDataReceived(short command, byte[] data);

        void onReceiveError(Exception e);
    }
    
    public interface DeviceCheckCallback {
        void onDeviceCheckResponse(ASR5515Protocol.DeviceInfo deviceInfo);
    }
    
    public interface DeviceControlCallback {
        void onDeviceRestartResponse(boolean success);
    }
    
    public interface LogControlCallback {
        void onLogControlResponse(boolean success);
    }
    
    public interface BtVersionCallback {
        void onBtVersionResponse(String version);
    }
    
    public interface BtUpgradeCallback {
        void onBtUpgradeResponse(ASR5515Protocol.BtUpgradeResponse response);
    }
    
    public interface HostStatusCallback {
        void onHostStatusResponse(ASR5515Protocol.HostStatusResponse response);
    }
    
    public interface BootBinCheckCallback {
        void onBootBinCheckResponse(ASR5515Protocol.BootBinCheckResponse response);
    }
    
    public interface CollectFreqSetCallback {
        void onCollectFreqSetResponse(ASR5515Protocol.CollectFreqSetResponse response);
    }
    
    public interface HrBpBoAutoMeasureCallback {
        void onHrBpBoAutoMeasureResponse(ASR5515Protocol.HrBpBoAutoMeasureResponse response);
    }
    
    public interface HrBpBoManualMeasureCallback {
        void onHrBpBoManualMeasureResponse(ASR5515Protocol.HrBpBoManualMeasureResponse response);
    }
    
    public interface DynamicMeasureCallback {
        void onDynamicMeasureResponse(ASR5515Protocol.DynamicMeasureResponse response);
    }

    public interface WearDetectionCallback {
        void onWearDetectionResponse(ASR5515Protocol.WearDetectionResponse response);
    }

    public SerialPortHelper(String portName, int baudRate) {
        this.mPortName = portName;
        this.mBaudRate = baudRate;
        this.isRunning = false;
        this.isSimulationMode = false;
    }
    
    public void setSimulationMode(boolean enabled) {
        this.isSimulationMode = enabled;
    }

    public void setSendCallback(SendCallback callback) {
        this.sendCallback = callback;
    }

    public void setReceiveCallback(ReceiveCallback callback) {
        this.receiveCallback = callback;
    }

    public void setDeviceCheckCallback(DeviceCheckCallback callback) {
        this.deviceCheckCallback = callback;
    }
    
    public void setDeviceControlCallback(DeviceControlCallback callback) {
        this.deviceControlCallback = callback;
    }

    public void setLogControlCallback(LogControlCallback callback) {
        this.logControlCallback = callback;
    }
    
    public void setBtVersionCallback(BtVersionCallback callback) {
        this.btVersionCallback = callback;
    }
    
    public void setBtUpgradeCallback(BtUpgradeCallback callback) {
        this.btUpgradeCallback = callback;
    }
    
    public void setHostStatusCallback(HostStatusCallback callback) {
        this.hostStatusCallback = callback;
    }

    public void setBootBinCheckCallback(BootBinCheckCallback callback) {
        this.bootBinCheckCallback = callback;
    }

    public void setCollectFreqSetCallback(CollectFreqSetCallback callback) {
        this.collectFreqSetCallback = callback;
    }
    
    public void setHrBpBoAutoMeasureCallback(HrBpBoAutoMeasureCallback callback) {
        this.hrBpBoAutoMeasureCallback = callback;
    }

    public void setHrBpBoManualMeasureCallback(HrBpBoManualMeasureCallback callback) {
        this.hrBpBoManualMeasureCallback = callback;
    }
    
    public void setDynamicMeasureCallback(DynamicMeasureCallback callback) {
        this.dynamicMeasureCallback = callback;
    }

    public void setWearDetectionCallback(WearDetectionCallback callback) {
        this.wearDetectionCallback = callback;
    }

    /**
     * 发送佩戴检测查询请求
     */
    public void sendWearDetectionRequest() {
        byte[] data = ASR5515Protocol.createWearDetectionRequest();
        sendData(data, null);
    }

    // ASR-5515协议命令发送方法
    public void checkDevice() {
        byte[] data = ASR5515Protocol.createDeviceCheckRequest();
        sendData(data, null);
    }

    public void restartDevice() {
        byte[] data = ASR5515Protocol.createDeviceRestartRequest();
        sendData(data, null);
    }

    public void setLogControl(boolean enable) {
        byte[] data = ASR5515Protocol.createLogControlRequest((byte)0, (byte)1, (byte)(enable ? 1 : 0));
        sendData(data, null);
    }

    public void queryBtVersion() {
        byte[] data = ASR5515Protocol.createBtVersionRequest();
        sendData(data, null);
    }

    /**
     * 发送蓝牙升级请求
     */
    public void upgradeBt() {
        byte[] data = ASR5515Protocol.createBtUpgradeRequest();
        sendData(data, null);
    }

    /**
     * 发送主机状态同步请求
     * @param screenOn 屏幕状态（true：亮屏，false：灭屏）
     */
    public void syncHostStatus(boolean screenOn) {
        byte[] data = ASR5515Protocol.createHostStatusRequest(screenOn);
        sendData(data, null);
    }

    /**
     * 发送boot bin检查请求
     */
    public void checkBootBin() {
        byte[] data = ASR5515Protocol.createBootBinCheckRequest();
        sendData(data, null);
    }

    /**
     * 设置采集频率
     * @param len 长度
     * @param freq 频率值
     */
    public void setCollectFreq(byte len, byte freq) {
        byte[] data = ASR5515Protocol.createCollectFreqSetRequest(len, freq);
        sendData(data, null);
    }

    /**
     * 发送HrBpBo自动测量请求
     */
    public void startHrBpBoAutoMeasure() {
        byte[] data = ASR5515Protocol.createHrBpBoAutoMeasureRequest();
        sendData(data, null);
    }

    /**
     * 发送HrBpBo手动测量请求
     */
    public void startHrBpBoManualMeasure() {
        byte[] data = ASR5515Protocol.createHrBpBoManualMeasureRequest();
        sendData(data, null);
    }

    /**
     * 发送动态测量请求
     */
    public void startDynamicMeasure() {
        byte[] data = ASR5515Protocol.createDynamicMeasureRequest();
        sendData(data, null);
    }

    public boolean open() {
        try {
            LogManager.d(TAG, "Opening serial port: " + mPortName + " with baud rate: " + mBaudRate);
            if (!isSimulationMode) {
                mSerialPort = new SerialPort(new File(mPortName), mBaudRate);
                LogManager.d(TAG, "Serial port opened successfully");
            } else {
                LogManager.d(TAG, "Running in simulation mode, skipping actual port opening");
            }
            startSendThread();
            startReceiveThread();
            startProcessThread();
            return true;
        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }
    }

    public void close() {
        isRunning = false;
        if (sendThread != null) {
            sendThread.interrupt();
            try {
                sendThread.join();
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
            }
            sendThread = null;
        }
        if (receiveThread != null) {
            receiveThread.interrupt();
            try {
                receiveThread.join();
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
            }
            receiveThread = null;
        }
        if (processThread != null) {
            processThread.interrupt();
            try {
                processThread.join();
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
            }
            processThread = null;
        }
        if (mSerialPort != null) {
            mSerialPort.closePort();
            mSerialPort = null;
        }
    }

    public void sendAsync(byte[] data) {
        if (data != null) {
            sendQueue.offer(data);
        }
    }

    public void sendData(byte[] data, SendCallback callback) {
        if (data == null || data.length == 0) {
            if (callback != null) {
                callback.onSendFailed(data, new IllegalArgumentException("Data is null or empty"));
            }
            return;
        }

        if (isSimulationMode) {

            if (callback != null) {
                callback.onSendSuccess(data);
            }
            return;
        }
//        LogManager.d(TAG, "ASR->5515 DATA: " + bytesToHexString(data));
        sendAsync(data);
        if (callback != null) {
            this.sendCallback = callback;
        }
    }
    
    /**
     * 将字节数组转换为16进制字符串
     * @param bytes 字节数组
     * @return 16进制字符串，格式如: "01 A2 FF"
     */
    public static String bytesToHexString(byte[] bytes) {
        if (bytes == null || bytes.length == 0) {
            return "";
        }
        StringBuilder sb = new StringBuilder();
        for (byte b : bytes) {
            sb.append(String.format("%02X ", b));
        }
        return sb.toString().trim();
    }


    private void startSendThread() {
        if (sendThread == null) {
            LogManager.d(TAG, "Starting send thread");
            isRunning = true;
            sendThread = new Thread(() -> {
                LogManager.d(TAG, "Send thread started");
                while (isRunning) {
                    try {
                        byte[] data = sendQueue.take();
                        if (isSimulationMode) {
                            LogManager.d(TAG, "[SimulationMode] Sending data: " + bytesToHexString(data));
                            if (sendCallback != null) {
                                sendCallback.onSendSuccess(data);
                            }
                        } else if (mSerialPort != null) {
                            try {
                                LogManager.d(TAG, "ASR->5515 DATA: " + Arrays.toString(data));
                                LogManager.d(TAG, "ASR->5515 DATA: " + bytesToHexString(data));
                                mSerialPort.getOutputStream().write(data);
                                if (sendCallback != null) {
                                    sendCallback.onSendSuccess(data);
                                }
                            } catch (IOException e) {
                                //LogManager.e(TAG, "Failed to send data: " + Arrays.toString(data));
                                if (sendCallback != null) {
                                    sendCallback.onSendFailed(data, e);
                                }
                            }
                        }
                    } catch (InterruptedException e) {
                        Thread.currentThread().interrupt();
                    }
                }
            });
            sendThread.start();
        }
    }

    private void startReceiveThread() {
        if (receiveThread == null) {
            receiveThread = new Thread(() -> {
                byte[] buffer = new byte[1024];
                while (isRunning) {
                    if (mSerialPort != null) {
                        try {
                            int bytesRead = mSerialPort.getInputStream().read(buffer);
                            if (bytesRead > 0) {
                                byte[] receivedData = Arrays.copyOf(buffer, bytesRead);
                                LogManager.d(TAG, "5515->ASR DATA: " + Arrays.toString(receivedData));
                                LogManager.d(TAG, "5515->ASR DATA: " + bytesToHexString(receivedData));
                                receiveQueue.put(receivedData);
                            }
                        } catch (IOException e) {
                            if (receiveCallback != null) {
                                receiveCallback.onReceiveError(e);
                            }
                        } catch (InterruptedException e) {
                            Thread.currentThread().interrupt();
                        }
                    }
                }
            });
            receiveThread.start();
        }
    }

    private void startProcessThread() {
        if (processThread == null) {
            processThread = new Thread(() -> {
                while (isRunning) {
                    try {
                        byte[] receivedData = receiveQueue.poll(100, TimeUnit.MILLISECONDS);
                        if (receivedData != null) {
                            handleReceivedData(receivedData);
                        }
                    } catch (InterruptedException e) {
                        Thread.currentThread().interrupt();
                    }
                }
            });
            processThread.start();
        }
    }

    public int write(byte[] buffer, int size) {
        if (mSerialPort != null) {
            try {
                mSerialPort.getOutputStream().write(buffer, 0, size);
                return size;  // 返回实际写入的字节数
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return -1;
    }

    public int read(byte[] buffer, int size) {
        if (mSerialPort != null) {
            try {
                return mSerialPort.getInputStream().read(buffer, 0, size);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return -1;
    }
    
    private void handleReceivedData(byte[] data) {
        // 将新数据添加到帧缓冲区
        if (frameBuffer.position() + data.length > frameBuffer.capacity()) {
            // 缓冲区溢出，清空缓冲区
            frameBuffer.clear();
            LogManager.e(TAG, "Frame buffer overflow, cleared");
            return;
        }
        frameBuffer.put(data);

        // 尝试解析完整的帧
        while (frameBuffer.position() > 0) {
            frameBuffer.flip(); // 切换到读取模式
            byte[] bufferArray = new byte[frameBuffer.remaining()];
            frameBuffer.get(bufferArray);
            frameBuffer.clear();

            ASR5515Protocol.Frame frame = ASR5515Protocol.parseFrame(bufferArray);
            if (frame == null) {
                // 解析失败，可能是数据不完整，保留数据等待下次处理
                frameBuffer.put(bufferArray);
                break;
            }

            // 处理解析出的帧
            processFrame(frame);

            // 如果还有剩余数据，放回缓冲区
            if (frame.remainingData != null && frame.remainingData.length > 0) {
                frameBuffer.put(frame.remainingData);
            }
        }
    }

    private void processFrame(ASR5515Protocol.Frame frame) {
        // 通知原始数据接收回调
        if (receiveCallback != null) {
            receiveCallback.onDataReceived(frame.command, frame.data);
        }
        LogManager.d(TAG,"CMD TYPE:"+frame.command);
        // 处理特定类型的响应
        switch (frame.command) {
            case ASR5515Protocol.Commands.CMD_CHECK_DEVICE_RESP:
                ASR5515Protocol.DeviceInfo deviceInfo = ASR5515Protocol.parseDeviceCheckResponse(frame);
                if (deviceInfo != null && deviceCheckCallback != null) {
                    deviceCheckCallback.onDeviceCheckResponse(deviceInfo);
                }
                break;

            case ASR5515Protocol.Commands.CMD_RESTART_DEVICE_RESP:
                if (deviceControlCallback != null) {
                    deviceControlCallback.onDeviceRestartResponse(frame.data.length == 0);
                }
                break;

            case ASR5515Protocol.Commands.CMD_LOG_CONTROL_RESP:
                if (logControlCallback != null) {
                    logControlCallback.onLogControlResponse(frame.data.length == 0);
                }
                break;

            case ASR5515Protocol.Commands.CMD_BT_VERSION_RESP:
                String version = ASR5515Protocol.parseBtVersionResponse(frame);
                if (version != null && btVersionCallback != null) {
                    btVersionCallback.onBtVersionResponse(version);
                }
                break;

            case ASR5515Protocol.Commands.CMD_BT_UPGRADE_RESP:
                ASR5515Protocol.BtUpgradeResponse upgradeResponse = ASR5515Protocol.parseBtUpgradeResponse(frame);
                if (upgradeResponse != null && btUpgradeCallback != null) {
                    btUpgradeCallback.onBtUpgradeResponse(upgradeResponse);
                }
                break;

            case ASR5515Protocol.Commands.CMD_HOST_STATUS_RESP:
            case ASR5515Protocol.Commands.CMD_HOST_STATUS_BOOT_RESP:
                ASR5515Protocol.HostStatusResponse statusResponse = ASR5515Protocol.parseHostStatusResponse(frame);
                if (statusResponse != null && hostStatusCallback != null) {
                    hostStatusCallback.onHostStatusResponse(statusResponse);
                }
                break;
                
            case ASR5515Protocol.Commands.CMD_BOOT_BIN_CHECK_RESP:
                ASR5515Protocol.BootBinCheckResponse bootBinCheckResponse = ASR5515Protocol.parseBootBinCheckResponse(frame);
                if (bootBinCheckResponse != null && bootBinCheckCallback != null) {
                    bootBinCheckCallback.onBootBinCheckResponse(bootBinCheckResponse);
                }
                break;
                
            case ASR5515Protocol.Commands.CMD_COLLECT_FREQ_SET_RESP:
                ASR5515Protocol.CollectFreqSetResponse collectFreqSetResponse = ASR5515Protocol.parseCollectFreqSetResponse(frame);
                if (collectFreqSetResponse != null && collectFreqSetCallback != null) {
                    collectFreqSetCallback.onCollectFreqSetResponse(collectFreqSetResponse);
                }
                break;
                
            case ASR5515Protocol.Commands.CMD_HRBPBO_AUTO_MEASURE_RESP:
                ASR5515Protocol.HrBpBoAutoMeasureResponse hrBpBoAutoMeasureResponse = ASR5515Protocol.parseHrBpBoAutoMeasureResponse(frame);
                if (hrBpBoAutoMeasureResponse != null && hrBpBoAutoMeasureCallback != null) {
                    hrBpBoAutoMeasureCallback.onHrBpBoAutoMeasureResponse(hrBpBoAutoMeasureResponse);
                }
                break;
                
            case ASR5515Protocol.Commands.CMD_HRBPBO_MANUAL_MEASURE_RESP:
                ASR5515Protocol.HrBpBoManualMeasureResponse hrBpBoManualMeasureResponse = ASR5515Protocol.parseHrBpBoManualMeasureResponse(frame);
                if (hrBpBoManualMeasureResponse != null && hrBpBoManualMeasureCallback != null) {
                    hrBpBoManualMeasureCallback.onHrBpBoManualMeasureResponse(hrBpBoManualMeasureResponse);
                }
                break;
                
            case ASR5515Protocol.Commands.CMD_DYNAMIC_MEASURE_RESP:
                ASR5515Protocol.DynamicMeasureResponse dynamicMeasureResponse = ASR5515Protocol.parseDynamicMeasureResponse(frame);
                if (dynamicMeasureResponse != null && dynamicMeasureCallback != null) {
                    dynamicMeasureCallback.onDynamicMeasureResponse(dynamicMeasureResponse);
                }
                break;
                
            case ASR5515Protocol.Commands.CMD_WEAR_DETECTION_RESP:
                ASR5515Protocol.WearDetectionResponse wearDetectionResponse = ASR5515Protocol.parseWearDetectionResponse(frame);
                if (wearDetectionResponse != null && wearDetectionCallback != null) {
                    wearDetectionCallback.onWearDetectionResponse(wearDetectionResponse);
                }
                break;
        }
    }
}