package com.ismart.ib86.common.utils;

import android.content.Context;
import android.util.Log;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.Locale;

public final class LogManager implements Thread.UncaughtExceptionHandler {
    private static LogManager instance = new LogManager();

    private LogManager() {
    } // 私有构造方法防止实例化

    private static boolean isLogToFile = false;
    private static String LOG_DIR = "logs";
    private static Context appContext;
    private static final long MAX_LOG_SIZE = 10 * 1024 * 1024; // 10MB
    private static final int MAX_LOG_FILES = 30;

    public static void init(Context context) {
        Thread.setDefaultUncaughtExceptionHandler(instance);
        appContext = context.getApplicationContext();
        LOG_DIR = appContext.getExternalFilesDir(null).getAbsolutePath() + "/logs";
    }

    private static final SimpleDateFormat FILE_DATE_FORMAT = new SimpleDateFormat("yyyyMMdd", Locale.getDefault());
    private static final SimpleDateFormat LOG_DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS",
            Locale.getDefault());

    public static void setLogToFile(boolean enable) {
        isLogToFile = enable;
    }

    private static void writeToFile(String tag, String level, String msg) {
        if (!isLogToFile)
            return;

        if (appContext == null) {
            Log.e("LogManager", "Context not initialized, call init() first");
            return;
        }

        try {
            File logDir = new File(LOG_DIR);
            String fullPath = logDir.getAbsolutePath();
            Log.d("LogPath", "日志存储路径: " + fullPath);
            if (!logDir.exists()) {
                logDir.mkdirs();
            }

            // 清理旧日志文件
            File[] logFiles = logDir.listFiles();
            if (logFiles != null && logFiles.length > MAX_LOG_FILES) {
                Arrays.sort(logFiles, (f1, f2) -> Long.compare(f1.lastModified(), f2.lastModified()));
                for (int i = 0; i < logFiles.length - MAX_LOG_FILES; i++) {
                    logFiles[i].delete();
                }
            }

            String fileName = "log_" + FILE_DATE_FORMAT.format(new Date()) + ".txt";
            File logFile = new File(logDir, fileName);
            
            // 检查当前日志文件大小
            if (logFile.exists() && logFile.length() > MAX_LOG_SIZE) {
                logFile.delete();
            }

            String threadInfo = "[" + Thread.currentThread().getId() + "/" + Thread.currentThread().getName() + "]";
            String deviceInfo = "[" + android.os.Build.MODEL + "/" + android.os.Build.VERSION.RELEASE + "]";
            String logContent = LOG_DATE_FORMAT.format(new Date()) + " " + level + "/" + tag + ": " + threadInfo + " "
                    + deviceInfo + " " + msg + "\n";

            FileOutputStream fos = new FileOutputStream(logFile, true);
            fos.write(logContent.getBytes());
            fos.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static int v(String tag, String msg) {
        writeToFile(tag, "VERBOSE", msg);
        return Log.v(tag, msg);
    }

    public static int d(String tag, String msg) {
        writeToFile(tag, "DEBUG", msg);
        return Log.d(tag, msg);
    }

    public static int i(String tag, String msg) {
        writeToFile(tag, "INFO", msg);
        return Log.i(tag, msg);
    }

    public static int w(String tag, String msg) {
        writeToFile(tag, "WARN", msg);
        return Log.w(tag, msg);
    }

    @Override
    public void uncaughtException(Thread thread, Throwable ex) {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        ex.printStackTrace(pw);
        String stackTrace = sw.toString();
        e("CRASH", "Uncaught exception in thread " + thread.getName() + ": " + ex.getMessage() + "\n" + stackTrace);
    }

    public static int e(String tag, String msg) {
        writeToFile(tag, "ERROR", msg);
        return Log.e(tag, msg);
    }
}